// generated by Fast Light User Interface Designer (fluid) version 1.0300

#ifndef tabs_h
#define tabs_h
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
extern Fl_Double_Window *foo_window;
#include <FL/Fl_Box.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Button.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Wizard.H>
extern Fl_Wizard *wWizard;
#include <stdlib.h>
#include <FL/Fl_Return_Button.H>
#endif
