//
// "$Id: about_panel.h 8306 2011-01-24 17:04:22Z matt $"
//
// About dialog for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

// generated by Fast Light User Interface Designer (fluid) version 1.0300

#ifndef about_panel_h
#define about_panel_h
#include <FL/Fl.H>
extern void show_help(const char *name); 
#include <FL/Fl_Double_Window.H>
extern Fl_Double_Window *about_panel;
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Return_Button.H>
Fl_Double_Window* make_about_panel();
extern unsigned char fluid_org_png[12690];
#endif

//
// End of "$Id: about_panel.h 8306 2011-01-24 17:04:22Z matt $".
//
