// generated by Fast Light User Interface Designer (fluid) version 1.0300

#ifndef resize_h
#define resize_h
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#endif
